@@ -231,7 +231,7 @@ struct _GstExifWriter
 struct _GstExifReader
 {
   GstTagList *taglist;
-  const GstBuffer *buffer;
+  GstBuffer *buffer;
   guint32 base_offset;
   gint byte_order;
 
@@ -454,11 +454,11 @@ static const GstExifTagMatch tag_map_gps[] = {
 /* GstExifReader functions */
 static void
 gst_exif_reader_init (GstExifReader * reader, gint byte_order,
-    const GstBuffer * buf, guint32 base_offset)
+    GstBuffer * buf, guint32 base_offset)
 {
   ensure_exif_tags ();
 
-  reader->taglist = gst_tag_list_new ();
+  reader->taglist = gst_tag_list_new_empty ();
   reader->buffer = buf;
   reader->base_offset = base_offset;
   reader->byte_order = byte_order;
@@ -515,7 +515,7 @@ gst_exif_reader_reset (GstExifReader * reader, gboolean return_taglist)
   }
 
   if (reader->taglist) {
-    gst_tag_list_free (reader->taglist);
+    gst_tag_list_unref (reader->taglist);
   }
 
   return ret;
@@ -550,7 +550,7 @@ gst_exif_writer_reset_and_get_buffer (GstExifWriter * writer)
   header = gst_byte_writer_reset_and_get_buffer (&writer->tagwriter);
   data = gst_byte_writer_reset_and_get_buffer (&writer->datawriter);
 
-  return gst_buffer_join (header, data);
+  return gst_buffer_append (header, data);
 }
 
 /*
@@ -614,31 +614,40 @@ gst_exif_writer_write_tag_header (GstExifWriter * writer,
     guint16 exif_tag, guint16 exif_type, guint32 count, guint32 offset,
     const guint32 * offset_data)
 {
+  gboolean handled = TRUE;
+
   GST_DEBUG ("Writing tag entry: id %x, type %u, count %u, offset %u",
       exif_tag, exif_type, count, offset);
 
   if (writer->byte_order == G_LITTLE_ENDIAN) {
-    gst_byte_writer_put_uint16_le (&writer->tagwriter, exif_tag);
-    gst_byte_writer_put_uint16_le (&writer->tagwriter, exif_type);
-    gst_byte_writer_put_uint32_le (&writer->tagwriter, count);
+    handled &= gst_byte_writer_put_uint16_le (&writer->tagwriter, exif_tag);
+    handled &= gst_byte_writer_put_uint16_le (&writer->tagwriter, exif_type);
+    handled &= gst_byte_writer_put_uint32_le (&writer->tagwriter, count);
     if (offset_data != NULL) {
-      gst_byte_writer_put_data (&writer->tagwriter, (guint8 *) offset_data, 4);
+      handled &=
+          gst_byte_writer_put_data (&writer->tagwriter, (guint8 *) offset_data,
+          4);
     } else {
-      gst_byte_writer_put_uint32_le (&writer->tagwriter, offset);
+      handled &= gst_byte_writer_put_uint32_le (&writer->tagwriter, offset);
     }
   } else if (writer->byte_order == G_BIG_ENDIAN) {
-    gst_byte_writer_put_uint16_be (&writer->tagwriter, exif_tag);
-    gst_byte_writer_put_uint16_be (&writer->tagwriter, exif_type);
-    gst_byte_writer_put_uint32_be (&writer->tagwriter, count);
+    handled &= gst_byte_writer_put_uint16_be (&writer->tagwriter, exif_tag);
+    handled &= gst_byte_writer_put_uint16_be (&writer->tagwriter, exif_type);
+    handled &= gst_byte_writer_put_uint32_be (&writer->tagwriter, count);
     if (offset_data != NULL) {
-      gst_byte_writer_put_data (&writer->tagwriter, (guint8 *) offset_data, 4);
+      handled &=
+          gst_byte_writer_put_data (&writer->tagwriter, (guint8 *) offset_data,
+          4);
     } else {
-      gst_byte_writer_put_uint32_be (&writer->tagwriter, offset);
+      handled &= gst_byte_writer_put_uint32_be (&writer->tagwriter, offset);
     }
   } else {
     g_assert_not_reached ();
   }
 
+  if (G_UNLIKELY (!handled))
+    GST_WARNING ("Error writing tag header");
+
   writer->tags_total++;
 }
 
@@ -646,26 +655,36 @@ static void
 gst_exif_writer_write_rational_data (GstExifWriter * writer, guint32 frac_n,
     guint32 frac_d)
 {
+  gboolean handled = TRUE;
+
   if (writer->byte_order == G_LITTLE_ENDIAN) {
-    gst_byte_writer_put_uint32_le (&writer->datawriter, frac_n);
-    gst_byte_writer_put_uint32_le (&writer->datawriter, frac_d);
+    handled &= gst_byte_writer_put_uint32_le (&writer->datawriter, frac_n);
+    handled &= gst_byte_writer_put_uint32_le (&writer->datawriter, frac_d);
   } else {
-    gst_byte_writer_put_uint32_be (&writer->datawriter, frac_n);
-    gst_byte_writer_put_uint32_be (&writer->datawriter, frac_d);
+    handled &= gst_byte_writer_put_uint32_be (&writer->datawriter, frac_n);
+    handled &= gst_byte_writer_put_uint32_be (&writer->datawriter, frac_d);
   }
+
+  if (G_UNLIKELY (!handled))
+    GST_WARNING ("Error writing rational data");
 }
 
 static void
 gst_exif_writer_write_signed_rational_data (GstExifWriter * writer,
     gint32 frac_n, gint32 frac_d)
 {
+  gboolean handled = TRUE;
+
   if (writer->byte_order == G_LITTLE_ENDIAN) {
-    gst_byte_writer_put_int32_le (&writer->datawriter, frac_n);
-    gst_byte_writer_put_int32_le (&writer->datawriter, frac_d);
+    handled &= gst_byte_writer_put_int32_le (&writer->datawriter, frac_n);
+    handled &= gst_byte_writer_put_int32_le (&writer->datawriter, frac_d);
   } else {
-    gst_byte_writer_put_int32_be (&writer->datawriter, frac_n);
-    gst_byte_writer_put_int32_be (&writer->datawriter, frac_d);
+    handled &= gst_byte_writer_put_int32_be (&writer->datawriter, frac_n);
+    handled &= gst_byte_writer_put_int32_be (&writer->datawriter, frac_d);
   }
+
+  if (G_UNLIKELY (!handled))
+    GST_WARNING ("Error writing signed rational data");
 }
 
 static void
@@ -771,7 +790,9 @@ write_exif_undefined_tag (GstExifWriter * writer, guint16 tag,
     offset = gst_byte_writer_get_size (&writer->datawriter);
     gst_exif_writer_write_tag_header (writer, tag, EXIF_TYPE_UNDEFINED,
         size, offset, NULL);
-    gst_byte_writer_put_data (&writer->datawriter, data, size);
+    if (!gst_byte_writer_put_data (&writer->datawriter, data, size)) {
+      GST_WARNING ("Error writing undefined tag");
+    }
   } else {
     /* small enough to go in the offset */
     memcpy ((guint8 *) & offset, data, size);
@@ -875,9 +896,12 @@ write_exif_undefined_tag_from_taglist (GstExifWriter * writer,
     const GstTagList * taglist, const GstExifTagMatch * exiftag)
 {
   const GValue *value;
-  const guint8 *data = NULL;
-  gint size = 0;
+  GstMapInfo info;
+  guint8 *data = NULL;
+  gsize size = 0;
   gint tag_size = gst_tag_list_get_tag_size (taglist, exiftag->gst_tag);
+  GstSample *sample = NULL;
+  GstBuffer *buf = NULL;
 
   if (tag_size != 1) {
     GST_WARNING ("Only the first item in the taglist will be serialized");
@@ -893,11 +917,12 @@ write_exif_undefined_tag_from_taglist (GstExifWriter * writer,
       size = strlen ((gchar *) data);   /* no need to +1, undefined doesn't require it */
       break;
     default:
-      if (G_VALUE_TYPE (value) == GST_TYPE_BUFFER) {
-        GstBuffer *buf = gst_value_get_buffer (value);
-
-        data = GST_BUFFER_DATA (buf);
-        size = GST_BUFFER_SIZE (buf);
+      if (G_VALUE_TYPE (value) == GST_TYPE_SAMPLE) {
+        sample = gst_value_get_sample (value);
+        buf = gst_sample_get_buffer (sample);
+        gst_buffer_map (buf, &info, GST_MAP_READ);
+        data = info.data;
+        size = info.size;
       } else {
         GST_WARNING ("Conversion from %s to raw data not supported",
             G_VALUE_TYPE_NAME (value));
@@ -905,10 +930,11 @@ write_exif_undefined_tag_from_taglist (GstExifWriter * writer,
       break;
   }
 
-  if (size == 0)
-    return;
+  if (size > 0)
+    write_exif_undefined_tag (writer, exiftag->exif_tag, data, size);
 
-  write_exif_undefined_tag (writer, exiftag->exif_tag, data, size);
+  if (buf)
+    gst_buffer_unmap (buf, &info);
 }
 
 static void
@@ -1070,6 +1096,7 @@ gst_exif_tag_rewrite_offsets (GstByteWriter * writer, gint byte_order,
   GstByteReader *reader;
   gint i;
   guint16 aux = G_MAXUINT16;
+  gboolean handled = TRUE;
 
   GST_LOG ("Rewriting tag entries offsets");
 
@@ -1077,9 +1104,9 @@ gst_exif_tag_rewrite_offsets (GstByteWriter * writer, gint byte_order,
 
   if (num_tags == -1) {
     if (byte_order == G_LITTLE_ENDIAN) {
-      gst_byte_reader_get_uint16_le (reader, &aux);
+      handled &= gst_byte_reader_get_uint16_le (reader, &aux);
     } else {
-      gst_byte_reader_get_uint16_be (reader, &aux);
+      handled &= gst_byte_reader_get_uint16_be (reader, &aux);
     }
     if (aux == G_MAXUINT16) {
       GST_WARNING ("Failed to read number of tags, won't rewrite offsets");
@@ -1147,11 +1174,13 @@ gst_exif_tag_rewrite_offsets (GstByteWriter * writer, gint byte_order,
     if (byte_size > 4 || tag_id == EXIF_GPS_IFD_TAG || tag_id == EXIF_IFD_TAG) {
       if (byte_order == G_LITTLE_ENDIAN) {
         if (gst_byte_reader_peek_uint32_le (reader, &cur_offset)) {
-          gst_byte_writer_put_uint32_le (writer, cur_offset + offset);
+          handled &=
+              gst_byte_writer_put_uint32_le (writer, cur_offset + offset);
         }
       } else {
         if (gst_byte_reader_peek_uint32_be (reader, &cur_offset)) {
-          gst_byte_writer_put_uint32_be (writer, cur_offset + offset);
+          handled &=
+              gst_byte_writer_put_uint32_be (writer, cur_offset + offset);
         }
       }
       GST_DEBUG ("Rewriting tag offset from %u to (%u + %u) %u",
@@ -1170,10 +1199,13 @@ gst_exif_tag_rewrite_offsets (GstByteWriter * writer, gint byte_order,
             NULL);
       }
     } else {
-      gst_byte_reader_skip (reader, 4);
+      handled &= gst_byte_reader_skip (reader, 4);
       GST_DEBUG ("No need to rewrite tag offset");
     }
   }
+  if (G_UNLIKELY (!handled))
+    GST_WARNING ("Error rewriting offsets");
+
   GST_LOG ("Done rewriting offsets");
 }
 
@@ -1188,6 +1220,8 @@ parse_exif_ascii_tag (GstExifReader * reader, const GstExifTagMatch * tag,
   GError *error = NULL;
 
   if (count > 4) {
+    GstMapInfo info;
+
     if (offset < reader->base_offset) {
       GST_WARNING ("Offset is smaller (%u) than base offset (%u)", offset,
           reader->base_offset);
@@ -1195,15 +1229,17 @@ parse_exif_ascii_tag (GstExifReader * reader, const GstExifTagMatch * tag,
     }
 
     real_offset = offset - reader->base_offset;
-    if (real_offset >= GST_BUFFER_SIZE (reader->buffer)) {
-      GST_WARNING ("Invalid offset %u for buffer of size %u, not adding tag %s",
-          real_offset, GST_BUFFER_SIZE (reader->buffer), tag->gst_tag);
+
+    gst_buffer_map (reader->buffer, &info, GST_MAP_READ);
+    if (real_offset >= info.size) {
+      GST_WARNING ("Invalid offset %u for buffer of size %" G_GSIZE_FORMAT
+          ", not adding tag %s", real_offset, info.size, tag->gst_tag);
+      gst_buffer_unmap (reader->buffer, &info);
       return;
     }
 
-    str =
-        g_strndup ((gchar *) (GST_BUFFER_DATA (reader->buffer) + real_offset),
-        count);
+    str = g_strndup ((gchar *) (info.data + real_offset), count);
+    gst_buffer_unmap (reader->buffer, &info);
   } else {
     str = g_strndup ((gchar *) offset_as_data, count);
   }
@@ -1283,6 +1319,8 @@ parse_exif_undefined_tag (GstExifReader * reader, const GstExifTagMatch * tag,
   guint32 real_offset;
 
   if (count > 4) {
+    GstMapInfo info;
+
     if (offset < reader->base_offset) {
       GST_WARNING ("Offset is smaller (%u) than base offset (%u)", offset,
           reader->base_offset);
@@ -1290,16 +1328,22 @@ parse_exif_undefined_tag (GstExifReader * reader, const GstExifTagMatch * tag,
     }
 
     real_offset = offset - reader->base_offset;
-    if (real_offset >= GST_BUFFER_SIZE (reader->buffer)) {
-      GST_WARNING ("Invalid offset %u for buffer of size %u, not adding tag %s",
-          real_offset, GST_BUFFER_SIZE (reader->buffer), tag->gst_tag);
+
+    gst_buffer_map (reader->buffer, &info, GST_MAP_READ);
+
+    if (real_offset >= info.size) {
+      GST_WARNING ("Invalid offset %u for buffer of size %" G_GSIZE_FORMAT
+          ", not adding tag %s", real_offset, info.size, tag->gst_tag);
+      gst_buffer_unmap (reader->buffer, &info);
       return;
     }
 
     /* +1 because it could be a string without the \0 */
     data = malloc (sizeof (guint8) * count + 1);
-    memcpy (data, GST_BUFFER_DATA (reader->buffer) + real_offset, count);
+    memcpy (data, info.data + real_offset, count);
     data[count] = 0;
+
+    gst_buffer_unmap (reader->buffer, &info);
   } else {
     data = malloc (sizeof (guint8) * count + 1);
     memcpy (data, (guint8 *) offset_as_data, count);
@@ -1307,18 +1351,17 @@ parse_exif_undefined_tag (GstExifReader * reader, const GstExifTagMatch * tag,
   }
 
   tagtype = gst_tag_get_type (tag->gst_tag);
-  if (tagtype == GST_TYPE_BUFFER) {
+  if (tagtype == GST_TYPE_SAMPLE) {
+    GstSample *sample;
     GstBuffer *buf;
 
-    buf = gst_buffer_new ();
-    GST_BUFFER_DATA (buf) = data;
-    GST_BUFFER_MALLOCDATA (buf) = data;
-    GST_BUFFER_SIZE (buf) = count;
+    buf = gst_buffer_new_wrapped (data, count);
     data = NULL;
 
+    sample = gst_sample_new (buf, NULL, NULL, NULL);
     gst_tag_list_add (reader->taglist, GST_TAG_MERGE_APPEND, tag->gst_tag,
-        buf, NULL);
-
+        sample, NULL);
+    gst_sample_unref (sample);
     gst_buffer_unref (buf);
   } else if (tagtype == G_TYPE_STRING) {
     gst_tag_list_add (reader->taglist, GST_TAG_MERGE_REPLACE, tag->gst_tag,
@@ -1339,6 +1382,7 @@ exif_reader_read_rational_tag (GstExifReader * exif_reader,
   guint32 real_offset;
   gint32 frac_n = 0;
   gint32 frac_d = 0;
+  GstMapInfo info;
 
   if (count > 1) {
     GST_WARNING ("Rationals with multiple entries are not supported");
@@ -1350,13 +1394,16 @@ exif_reader_read_rational_tag (GstExifReader * exif_reader,
   }
 
   real_offset = offset - exif_reader->base_offset;
-  if (real_offset >= GST_BUFFER_SIZE (exif_reader->buffer)) {
-    GST_WARNING ("Invalid offset %u for buffer of size %u",
-        real_offset, GST_BUFFER_SIZE (exif_reader->buffer));
-    return FALSE;
+
+  gst_buffer_map (exif_reader->buffer, &info, GST_MAP_READ);
+
+  if (real_offset >= info.size) {
+    GST_WARNING ("Invalid offset %u for buffer of size %" G_GSIZE_FORMAT,
+        real_offset, info.size);
+    goto reader_fail;
   }
 
-  gst_byte_reader_init_from_buffer (&data_reader, exif_reader->buffer);
+  gst_byte_reader_init (&data_reader, info.data, info.size);
   if (!gst_byte_reader_set_pos (&data_reader, real_offset))
     goto reader_fail;
 
@@ -1390,10 +1437,13 @@ exif_reader_read_rational_tag (GstExifReader * exif_reader,
   if (_frac_d)
     *_frac_d = frac_d;
 
+  gst_buffer_unmap (exif_reader->buffer, &info);
+
   return TRUE;
 
 reader_fail:
   GST_WARNING ("Failed to read from byte reader. (Buffer too short?)");
+  gst_buffer_unmap (exif_reader->buffer, &info);
   return FALSE;
 }
 
@@ -1444,6 +1494,7 @@ write_exif_ifd (const GstTagList * taglist, guint byte_order,
 {
   GstExifWriter writer;
   gint i;
+  gboolean handled = TRUE;
 
   GST_DEBUG ("Formatting taglist %p as exif buffer. Byte order: %d, "
       "base_offset: %u", taglist, byte_order, base_offset);
@@ -1458,7 +1509,7 @@ write_exif_ifd (const GstTagList * taglist, guint byte_order,
   gst_exif_writer_init (&writer, byte_order);
 
   /* write tag number as 0 */
-  gst_byte_writer_put_uint16_le (&writer.tagwriter, 0);
+  handled &= gst_byte_writer_put_uint16_le (&writer.tagwriter, 0);
 
   /* write both tag headers and data
    * in ascending id order */
@@ -1493,12 +1544,17 @@ write_exif_ifd (const GstTagList * taglist, guint byte_order,
       }
 
       if (inner_ifd) {
+        GstMapInfo info;
+
         GST_DEBUG ("Adding inner ifd: %x", tag_map[i].exif_tag);
         gst_exif_writer_write_tag_header (&writer, tag_map[i].exif_tag,
             EXIF_TYPE_LONG, 1,
             gst_byte_writer_get_size (&writer.datawriter), NULL);
-        gst_byte_writer_put_data (&writer.datawriter,
-            GST_BUFFER_DATA (inner_ifd), GST_BUFFER_SIZE (inner_ifd));
+
+        gst_buffer_map (inner_ifd, &info, GST_MAP_READ);
+        handled &=
+            gst_byte_writer_put_data (&writer.datawriter, info.data, info.size);
+        gst_buffer_unmap (inner_ifd, &info);
         gst_buffer_unref (inner_ifd);
       }
       continue;
@@ -1514,14 +1570,16 @@ write_exif_ifd (const GstTagList * taglist, guint byte_order,
   /* Add the next IFD offset, we just set it to 0 because
    * there is no easy way to predict what it is going to be.
    * The user might rewrite the value if needed */
-  gst_byte_writer_put_uint32_le (&writer.tagwriter, 0);
+  handled &= gst_byte_writer_put_uint32_le (&writer.tagwriter, 0);
 
   /* write the number of tags */
   gst_byte_writer_set_pos (&writer.tagwriter, 0);
   if (writer.byte_order == G_LITTLE_ENDIAN)
-    gst_byte_writer_put_uint16_le (&writer.tagwriter, writer.tags_total);
+    handled &=
+        gst_byte_writer_put_uint16_le (&writer.tagwriter, writer.tags_total);
   else
-    gst_byte_writer_put_uint16_be (&writer.tagwriter, writer.tags_total);
+    handled &=
+        gst_byte_writer_put_uint16_be (&writer.tagwriter, writer.tags_total);
 
   GST_DEBUG ("Number of tags rewritten to %d", writer.tags_total);
 
@@ -1530,6 +1588,12 @@ write_exif_ifd (const GstTagList * taglist, guint byte_order,
       base_offset + gst_byte_writer_get_size (&writer.tagwriter),
       writer.tags_total, &writer.datawriter);
 
+  if (G_UNLIKELY (!handled)) {
+    GST_WARNING ("Error rewriting tags");
+    gst_buffer_unref (gst_exif_writer_reset_and_get_buffer (&writer));
+    return NULL;
+  }
+
   return gst_exif_writer_reset_and_get_buffer (&writer);
 }
 
@@ -1568,15 +1632,16 @@ parse_exif_ifd (GstExifReader * exif_reader, gint buf_offset,
   GstByteReader reader;
   guint16 entries = 0;
   guint16 i;
+  GstMapInfo info;
 
   g_return_val_if_fail (exif_reader->byte_order == G_LITTLE_ENDIAN
       || exif_reader->byte_order == G_BIG_ENDIAN, FALSE);
 
-  gst_byte_reader_init_from_buffer (&reader, exif_reader->buffer);
-  if (!gst_byte_reader_set_pos (&reader, buf_offset)) {
-    GST_WARNING ("Buffer offset invalid when parsing exif ifd");
-    return FALSE;
-  }
+  gst_buffer_map (exif_reader->buffer, &info, GST_MAP_READ);
+
+  gst_byte_reader_init (&reader, info.data, info.size);
+  if (!gst_byte_reader_set_pos (&reader, buf_offset))
+    goto invalid_offset;
 
   /* read the IFD entries number */
   if (exif_reader->byte_order == G_LITTLE_ENDIAN) {
@@ -1687,12 +1752,20 @@ parse_exif_ifd (GstExifReader * exif_reader, gint buf_offset,
       }
     }
   }
+  gst_buffer_unmap (exif_reader->buffer, &info);
 
   return TRUE;
 
+invalid_offset:
+  {
+    GST_WARNING ("Buffer offset invalid when parsing exif ifd");
+    gst_buffer_unmap (exif_reader->buffer, &info);
+    return FALSE;
+  }
 read_error:
   {
     GST_WARNING ("Failed to parse the exif ifd");
+    gst_buffer_unmap (exif_reader->buffer, &info);
     return FALSE;
   }
 }
@@ -1707,8 +1780,6 @@ read_error:
  * the tags IFD and is followed by the data pointed by the tag entries.
  *
  * Returns: A GstBuffer containing the tag entries followed by the tag data
- *
- * Since: 0.10.30
  */
 GstBuffer *
 gst_tag_list_to_exif_buffer (const GstTagList * taglist, gint byte_order,
@@ -1725,47 +1796,57 @@ gst_tag_list_to_exif_buffer (const GstTagList * taglist, gint byte_order,
  * is put in the beginning of the buffer.
  *
  * Returns: A GstBuffer containing the data
- *
- * Since: 0.10.30
  */
 GstBuffer *
 gst_tag_list_to_exif_buffer_with_tiff_header (const GstTagList * taglist)
 {
-  GstBuffer *ifd;
+  GstBuffer *ifd, *res;
   GstByteWriter writer;
-  guint size;
+  GstMapInfo info;
+  gboolean handled = TRUE;
 
   ifd = gst_tag_list_to_exif_buffer (taglist, G_BYTE_ORDER, 8);
   if (ifd == NULL) {
     GST_WARNING ("Failed to create exif buffer");
     return NULL;
   }
-  size = TIFF_HEADER_SIZE + GST_BUFFER_SIZE (ifd);
+
+  gst_buffer_map (ifd, &info, GST_MAP_READ);
 
   /* TODO what is the correct endianness here? */
-  gst_byte_writer_init_with_size (&writer, size, FALSE);
+  gst_byte_writer_init_with_size (&writer, info.size + TIFF_HEADER_SIZE, FALSE);
   /* TIFF header */
   if (G_BYTE_ORDER == G_LITTLE_ENDIAN) {
-    gst_byte_writer_put_uint16_le (&writer, TIFF_LITTLE_ENDIAN);
-    gst_byte_writer_put_uint16_le (&writer, 42);
-    gst_byte_writer_put_uint32_le (&writer, 8);
+    handled &= gst_byte_writer_put_uint16_le (&writer, TIFF_LITTLE_ENDIAN);
+    handled &= gst_byte_writer_put_uint16_le (&writer, 42);
+    handled &= gst_byte_writer_put_uint32_le (&writer, 8);
   } else {
-    gst_byte_writer_put_uint16_be (&writer, TIFF_BIG_ENDIAN);
-    gst_byte_writer_put_uint16_be (&writer, 42);
-    gst_byte_writer_put_uint32_be (&writer, 8);
+    handled &= gst_byte_writer_put_uint16_be (&writer, TIFF_BIG_ENDIAN);
+    handled &= gst_byte_writer_put_uint16_be (&writer, 42);
+    handled &= gst_byte_writer_put_uint32_be (&writer, 8);
   }
-  if (!gst_byte_writer_put_data (&writer, GST_BUFFER_DATA (ifd),
-          GST_BUFFER_SIZE (ifd))) {
+  if (!gst_byte_writer_put_data (&writer, info.data, info.size)) {
     GST_WARNING ("Byte writer size mismatch");
     /* reaching here is a programming error because we should have a buffer
      * large enough */
     g_assert_not_reached ();
+    gst_buffer_unmap (ifd, &info);
     gst_buffer_unref (ifd);
     gst_byte_writer_reset (&writer);
     return NULL;
   }
+  gst_buffer_unmap (ifd, &info);
   gst_buffer_unref (ifd);
-  return gst_byte_writer_reset_and_get_buffer (&writer);
+
+  res = gst_byte_writer_reset_and_get_buffer (&writer);
+
+  if (G_UNLIKELY (!handled)) {
+    GST_WARNING ("Error creating buffer");
+    gst_buffer_unref (res);
+    res = NULL;
+  }
+
+  return res;
 }
 
 /**
@@ -1780,11 +1861,9 @@ gst_tag_list_to_exif_buffer_with_tiff_header (const GstTagList * taglist)
  * start
  *
  * Returns: The parsed taglist
- *
- * Since: 0.10.30
  */
 GstTagList *
-gst_tag_list_from_exif_buffer (const GstBuffer * buffer, gint byte_order,
+gst_tag_list_from_exif_buffer (GstBuffer * buffer, gint byte_order,
     guint32 base_offset)
 {
   GstExifReader reader;
@@ -1813,11 +1892,9 @@ read_error:
  * Parses the exif tags starting with a tiff header structure.
  *
  * Returns: The taglist
- *
- * Since: 0.10.30
  */
 GstTagList *
-gst_tag_list_from_exif_buffer_with_tiff_header (const GstBuffer * buffer)
+gst_tag_list_from_exif_buffer_with_tiff_header (GstBuffer * buffer)
 {
   GstByteReader reader;
   guint16 fortytwo = 42;
@@ -1825,11 +1902,14 @@ gst_tag_list_from_exif_buffer_with_tiff_header (const GstBuffer * buffer)
   guint32 offset;
   GstTagList *taglist = NULL;
   GstBuffer *subbuffer;
+  GstMapInfo info, sinfo;
 
-  GST_LOG ("Parsing exif tags with tiff header of size %u",
-      GST_BUFFER_SIZE (buffer));
+  gst_buffer_map (buffer, &info, GST_MAP_READ);
 
-  gst_byte_reader_init_from_buffer (&reader, buffer);
+  GST_LOG ("Parsing exif tags with tiff header of size %" G_GSIZE_FORMAT,
+      info.size);
+
+  gst_byte_reader_init (&reader, info.data, info.size);
 
   GST_LOG ("Parsing the tiff header");
   if (!gst_byte_reader_get_uint16_be (&reader, &endianness)) {
@@ -1844,32 +1924,43 @@ gst_tag_list_from_exif_buffer_with_tiff_header (const GstBuffer * buffer)
     if (!gst_byte_reader_get_uint16_be (&reader, &fortytwo) ||
         !gst_byte_reader_get_uint32_be (&reader, &offset))
       goto byte_reader_fail;
-  } else {
-    GST_WARNING ("Invalid endianness number %u", endianness);
-    return NULL;
-  }
+  } else
+    goto invalid_endianness;
 
-  if (fortytwo != 42) {
-    GST_WARNING ("Invalid magic number %u, should be 42", fortytwo);
-    return NULL;
-  }
+  if (fortytwo != 42)
+    goto invalid_magic;
 
-  subbuffer = gst_buffer_new_and_alloc (GST_BUFFER_SIZE (buffer) -
-      (TIFF_HEADER_SIZE - 2));
-  memcpy (GST_BUFFER_DATA (subbuffer),
-      GST_BUFFER_DATA (buffer) + TIFF_HEADER_SIZE,
-      GST_BUFFER_SIZE (buffer) - TIFF_HEADER_SIZE);
+  subbuffer = gst_buffer_new_and_alloc (info.size - (TIFF_HEADER_SIZE - 2));
+
+  gst_buffer_map (subbuffer, &sinfo, GST_MAP_WRITE);
+  memcpy (sinfo.data, info.data + TIFF_HEADER_SIZE,
+      info.size - TIFF_HEADER_SIZE);
+  gst_buffer_unmap (subbuffer, &sinfo);
 
   taglist = gst_tag_list_from_exif_buffer (subbuffer,
       endianness == TIFF_LITTLE_ENDIAN ? G_LITTLE_ENDIAN : G_BIG_ENDIAN, 8);
 
   gst_buffer_unref (subbuffer);
+
+done:
+  gst_buffer_unmap (buffer, &info);
+
   return taglist;
 
 byte_reader_fail:
   {
     GST_WARNING ("Failed to read values from buffer");
-    return NULL;
+    goto done;
+  }
+invalid_endianness:
+  {
+    GST_WARNING ("Invalid endianness number %u", endianness);
+    goto done;
+  }
+invalid_magic:
+  {
+    GST_WARNING ("Invalid magic number %u, should be 42", fortytwo);
+    goto done;
   }
 }
 
@@ -1969,6 +2060,7 @@ deserialize_geo_coordinate (GstExifReader * exif_reader,
   gdouble degrees;
   gdouble minutes;
   gdouble seconds;
+  GstMapInfo info = { NULL };
 
   GST_LOG ("Starting to parse %s tag in exif 0x%x", exiftag->gst_tag,
       exiftag->exif_tag);
@@ -2025,8 +2117,11 @@ deserialize_geo_coordinate (GstExifReader * exif_reader,
     return ret;
   }
 
+  gst_buffer_map (exif_reader->buffer, &info, GST_MAP_READ);
+
   /* now parse the fractions */
-  gst_byte_reader_init_from_buffer (&fractions_reader, exif_reader->buffer);
+  gst_byte_reader_init (&fractions_reader, info.data, info.size);
+
   if (!gst_byte_reader_set_pos (&fractions_reader,
           next_tagdata.offset - exif_reader->base_offset))
     goto reader_fail;
@@ -2048,6 +2143,7 @@ deserialize_geo_coordinate (GstExifReader * exif_reader,
         !gst_byte_reader_get_uint32_be (&fractions_reader, &seconds_d))
       goto reader_fail;
   }
+  gst_buffer_unmap (exif_reader->buffer, &info);
 
   GST_DEBUG ("Read degrees fraction for tag %s: %u/%u %u/%u %u/%u",
       exiftag->gst_tag, degrees_n, degrees_d, minutes_n, minutes_d,
@@ -2069,6 +2165,8 @@ deserialize_geo_coordinate (GstExifReader * exif_reader,
 
 reader_fail:
   GST_WARNING ("Failed to read fields from buffer (too short?)");
+  if (info.data)
+    gst_buffer_unmap (exif_reader->buffer, &info);
   return ret;
 }
 
